/* RPO-04-FSM.c 24.03.2015
*
* pic18F87J11
*
* FSM Moore - function on background
* FSM Moore - function called on interrupt
* FSM Moore - direct code on interrupt
*
* LED toggle control on background
*
* Timer0, interrupt high priority
* Timer1, interrupt low priority
*/


//Includes
#include <p18F87J11.h>

//New type definitions
typedef int Tboolean;

//Function prototypes
Tboolean synchronousFsmMoore(Tboolean input);
void synchronousFsmMooreA(void);
void Interrupt_low (void);
void Interrupt_high (void);
void Timer0_int (void);
void Timer1_int (void);

//Constants
enum {FALSE, TRUE};

//Global variables
Tboolean timeXmsTick = FALSE;
Tboolean timeYmsTick = FALSE;

void main(void){
	Tboolean inputValue = TRUE;
	Tboolean ledToggleEnable = FALSE;

	// Timer 0
	T0CON &= 0b11011111;			// TMR0 internal clock
	INTCON2bits.TMR0IP = 1;		// TMR0 high priority interrupt group
	INTCONbits.TMR0IE=1;			// Enable TMR0 interrupt

	// Timer 1
	T1CONbits.RD16 = 1;
	T1CONbits.TMR1ON = 1; 		// Enable Timer1
	TMR1H = 0;
	TMR1L = 0;
	IPR1bits.TMR1IP = 0;			// Low priority interrupt
	PIE1bits.TMR1IE = 1; 			// Enable Timer1 interrupt

	RCONbits.IPEN = 1;				// Enable priority feature
	INTCONbits.GIEH = 1;			// Enable high priority interrupts
	INTCONbits.GIEL = 1; 			// Enable low prioritz interrupts

	while(TRUE){
	
		if(timeXmsTick){
			timeXmsTick = FALSE;
 			//volani metody pro odkmitani tlacitka
			ledToggleEnable = synchronousFsmMoore(inputValue);
			//volani metody pro blikani
		}
	}
}//main()END


Tboolean synchronousFsmMoore(Tboolean input){
	enum {STATE0, STATE1, STATE2, STATE3};
	static int fsmStateCtr = STATE0;

	switch(fsmStateCtr){
		case STATE0:{
			//State transition logic here
			// if(input == TRUE){}
			fsmStateCtr = STATE1;
			break;
		}
		case STATE1:{
			//State transition logic here
			fsmStateCtr = STATE2;
			break;
		}
		case STATE2:{
			//State transition logic here
			fsmStateCtr = STATE3;
			break;
		}
		case STATE3:{
			//State transition logic here
			fsmStateCtr = STATE0;
			break;
		}
		default:{
			//Error, never goes here
			fsmStateCtr = STATE0;
		}

		return TRUE;
	}//switch()END
}//fsm() END

void synchronousFsmMooreA(void){
	enum {INIT, STOP, START, RUNNING};
	static int fsmStateCtrA = INIT;

	switch(fsmStateCtrA){
		case STOP:{
			//State transition logic here
			// if(input == TRUE){}
			fsmStateCtrA = START;
			break;
		}
		case START:{
			//State transition logic here
			fsmStateCtrA = RUNNING;
			break;
		}
		case RUNNING:{
			//State transition logic here
			fsmStateCtrA = STOP;
			break;
		}
		case INIT:{
			//State transition logic here
			fsmStateCtrA = STOP;
		}
		default:{
			//Error, never goes here
			fsmStateCtrA = STOP;
		}
	}//switch()END
	
}// synchronousFsmMooreA() END


#pragma interrupt Timer0_int
void Timer0_int (void) /////////////////////////////////////////////////////////
{
	INTCONbits.TMR0IF=0;
	timeXmsTick = TRUE;

}// Timer0_int() END ///////////////////////////////////////////////////////////

	//enum {STOP, START, RUNNING};

#pragma interruptlow Timer1_int
void Timer1_int (void) /////////////////////////////////////////////////////////
{
	// Synchronous FSM Moore
	enum {STOP, START, RUNNING};
	static int fsmStateCtrA = STOP;

	switch(fsmStateCtrA){
		case STOP:{
			//State transition logic here
			// if(??){}
			fsmStateCtrA = START;
			break;
		}
		case START:{
			//State transition logic here
			fsmStateCtrA = RUNNING;
			break;
		}
		case RUNNING:{
			//State transition logic here
			fsmStateCtrA = STOP;
			break;
		}
		default:{
			//Error, never goes here
			fsmStateCtrA = STOP;
		}
	}//switch()END
	// FSM Moore END
	
	synchronousFsmMooreA();

	PIR1bits.TMR1IF = 0;	// Cleare interrupt flag
}// Timer1_int() END ///////////////////////////////////////////////////////////


#pragma code int_high=0x000008
void Interrupt_high (void) /////////////////////////////////////////////////////
{
	_asm GOTO Timer0_int _endasm
}// Interrupt_high() END ///////////////////////////////////////////////////////


#pragma code int_low=0x000018
void Interrupt_low (void) //////////////////////////////////////////////////////
{
	_asm GOTO Timer1_int _endasm
}// Interrupt_low() END ////////////////////////////////////////////////////////
